library PHPInspector;

{
  PHP Inspector Plugin for AlphaCode (c) 2004 - 2005 by David Fekete,
  AlphaCode (c) 2002 - 2005 by David Fekete,
  AlphaCode Plugin Interface (c) 2003 - 2005 by David Fekete,

  All rights reserved.
}

uses
  AlphaCodePluginInterface_TLB,
  Windows,
  Messages,
  CommCtrl,
  uPHPInspector;

{$R *.res}

type
  TPlugin = class;
  THandler = class;

  TPlugin = class(TInterfacedObject, IPlugin)
  private
    Handler: THandler;
  public
    constructor Create(Handler: THandler);
    function GetName: WideString; stdcall;
    function GetAuthor: WideString; stdcall;
    function GetComment: WideString; stdcall;
    function EventHandler(EventNotification: TEventNotification): TEventAction;
      stdcall;
  end;

  THandler = class(TObject)
  private
    FPage: IPageControlPage;
    FApp: IApplication;
    FPopUp: HMENU;
    FTree: HWND;
    FBar : HWND;
    FToolButton: TTBButton;
    FBarHeight: Integer;
    FFont: HFONT;
    FOldWndProc: Pointer;
    FOldPageControlProc: Pointer;
    FPHPSource: TPHPSource;
  public
    procedure Update(const Sender: IUnknown);
    constructor Create(const App: IApplication);
    destructor Destroy; override;
end;

// need to store the current in a variable because the wnd proc function
// is a regular procedure outside the class!
var
  FHandler: THandler;

// ID for the Toolbutton
const
  TOOLBUTTONIDC = 120;

{ TPlugin }

constructor TPlugin.Create(Handler: THandler);
begin
  Self.Handler := Handler;
end;

function TPlugin.GetName: WideString;
begin
  Result := 'AlphaCode PHP Inspektor';
end;

function TPlugin.GetAuthor: WideString;
begin
  Result := #169 + ' 2004 - 2005 by David Fekete';
end;

function TPlugin.GetComment: WideString;
begin
  Result := 'PHP Inspektor Plugin fr AlphaCode';
end;

function TPlugin.EventHandler(EventNotification: TEventNotification
  ): TEventAction;
begin
  Result := evnContinue;
  // Update TreeView on specific events, set the size of the TreeView
  // when the Quickbar is resized
  case EventNotification of
    evnNewFile, evnOpenFile, evnSaveFile, evnReloadFile, evnDocumentSelected,
    evnCloseFile, evnCloseAll, evnQuickbarPageChanged:
      if (FHandler.FApp.GetGUI.GetQuickbar.GetActivePage.GetName = FHandler.FPage.GetName) and
        (FHandler.FApp.GetGUI.GetQuickbar.GetVisible) then
        Handler.Update(nil);
    evnQuickbarResized: SetWindowPos(Handler.FTree, Handler.FPage.GetHandle,
      0, Handler.FBarHeight, Handler.FPage.GetWidth,
      Handler.FPage.GetHeight - Handler.FBarHeight, SWP_NOMOVE or SWP_NOZORDER);
  end;
end;

{ THandler }

function TreeWndProc(hTree, uMsg, wParam, lParam: DWORD): DWORD; stdcall;
var
  HitTest: TTVHitTestInfo;
  Point: TPoint;
  tv: tagTVITEM;
begin
  case uMsg of
    WM_RBUTTONDOWN, WM_CONTEXTMENU:
    begin
      if wParam = FHandler.FTree then
      begin
        // show popup menu
        Point.X := LoWord(lParam);
        Point.Y := HiWord(lParam);
        if (Point.X = -1) or (Point.Y = -1) or
          (Point.X > GetSystemMetrics(SM_CXSCREEN)) or
          (Point.Y > GetSystemMetrics(SM_CYSCREEN)) then
          GetCursorPos(Point);
        TrackPopupMenu(FHandler.FPopUp, 0, Point.X, Point.Y, 0,
          FHandler.FTree, nil);
      end;
        if FHandler.FApp.CanEditDocuments then
          SetFocus(FHandler.FApp.GetActiveDocument.GetEditor.GetHandle);
      Result := 0;
    end;
    WM_LBUTTONDOWN:
    begin
      // redirect to the original window proc so that items gets selected
      Result := CallWindowProc(FHandler.FOldWndProc, hTree, uMsg, wParam,
        lParam);
      if not FHandler.FApp.CanEditDocuments then Exit;
      // test if an item has been selected
      // if yes then then look if there is a stored cursor pos in the lParam field
      // and set the selstart in the editor
      GetCursorPos(Point);
      ScreenToClient(FHandler.FTree, Point);
      with HitTest do
      begin
        pt.X := Point.X;
        pt.Y := Point.Y;
        if TreeView_HitTest(FHandler.FTree, HitTest) <> nil then
        begin
          tv.mask := TVIF_PARAM;
          tv.hItem := hItem;
          if TreeView_GetItem(FHandler.FTree, tv) and
            (FHandler.FApp.CanEditDocuments) and (tv.lParam <> -1) then
            FHandler.FApp.GetActiveDocument.GetEditor.GetTextAttributes.SetSelStart(tv.lParam);
        end;
      end;
      // set window focus anyway, else there's an error with editor focus
      SetFocus(FHandler.FApp.GetActiveDocument.GetEditor.GetHandle);
    end;
    WM_COMMAND:
    begin
      // catch click of the popup menu
      if HiWord(wParam) = BN_CLICKED then
      begin
        if (LoWord(lParam) = 0) then
          FHandler.Update(nil);
      end;
      Result := 0;      
    end;
    else Result := CallWindowProc(FHandler.FOldWndProc, hTree, uMsg, wParam,
      lParam);
  end;
end;

function ToolbarWndProc(hToolbar, uMsg, wParam, lParam: DWORD): DWORD; stdcall;
begin
  case uMsg of
    WM_COMMAND:
      begin
        if HiWord(wParam) = BN_CLICKED then
          case LoWord(wParam) of
            TOOLBUTTONIDC: FHandler.Update(nil);
          end;
      end;
    WM_NOTIFY:
      begin
        case PNMToolBar(lParam)^.hdr.code of
          TTN_NEEDTEXT:
            case PToolTipText(lParam).hdr.idFrom of
              TOOLBUTTONIDC:
                PToolTipText(lParam).lpszText := 'Ansicht aktualisieren';
            end;
        end;
      end;
  end;
  // Don't forget to call the original Window Proc of the Page Control!
  Result := CallWindowProc(FHandler.FOldPageControlProc, hToolbar, uMsg, wParam,
    lParam);
end;

constructor THandler.Create(const App: IApplication);
var
  BarRect: TRect;
begin
  FHandler := Self;
  FApp := App;
  FPHPSource := TPHPSource.Create('');
  // create a new page in the quickbar
  FPage := FApp.GetGUI.GetQuickbar.CreatePage('pg_Quickbar_PHPInspector',
    'PHP Inspektor');
  FPage.Set_ImageIndex(58);
  // create toolbutton
  with FToolButton do
  begin
    iBitmap := 45;
    idCommand := TOOLBUTTONIDC;
    fsState := TBSTATE_ENABLED;
    fsStyle := BTNS_BUTTON;
    dwData := 0;
    iString := 0;
  end;
  // create toolbar
  FBar := CreateWindowEx(0, TOOLBARCLASSNAME, nil, WS_CHILD or
    WS_VISIBLE or CCS_NODIVIDER or TBSTYLE_TOOLTIPS or TBSTYLE_FLAT,
    0, 0, 0, 0, FPage.GetHandle, 101, hInstance, nil);
  SendMessage(FBar, TB_BUTTONSTRUCTSIZE, SizeOf(TTBBUTTON), 0);
  SendMessage(FBar, TB_ADDBUTTONS, 1, LPARAM(@FToolButton));
  // set imagelist
  SendMessage(FBar, TB_SETIMAGELIST, 0,
    FApp.GetMainMenu.GetImageList.GetHandle);
  GetWindowRect(FBar, BarRect);
  FBarHeight := BarRect.Bottom - BarRect.Top;
  // redirect bar window proc of the page control (because toolbutton messages
  // are only sent to the owner window of the toolbar, so we need to use this)
  FOldPageControlProc := Pointer(SetWindowLong(FPage.GetHandle, GWL_WNDPROC,
    Integer(@ToolbarWndProc)));
  // create the treeview
  FTree := CreateWindowEx(WS_EX_CLIENTEDGE, WC_TREEVIEW, nil,
    WS_VISIBLE or WS_CHILD or TVS_HASLINES or TVS_LINESATROOT
    or TVS_HASBUTTONS, 0, FBarHeight, FPage.GetWidth,
    FPage.GetHeight - FBarHeight, FPage.GetHandle, 0, hInstance, nil);
  FFont := CreateFont(-11, 0, 0, 0, 0, 0, 0, 0, ANSI_CHARSET,
    OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
    DEFAULT_PITCH, 'Tahoma');
  SendMessage(FTree, WM_SETFONT, Integer(FFont), Integer(True));
  // redirect the wnd proc
  FOldWndProc := Pointer(SetWindowLong(FTree, GWL_WNDPROC,
    Integer(@TreeWndProc)));
  // create pop up menu
  FPopUp := CreatePopUpMenu;
  AppendMenu(FPopup, MF_STRING, 0, 'Aktualisieren');
end;

destructor THandler.Destroy;
begin
  if FTree <> 0 then
    DestroyWindow(FTree);
  if FBar <> 0 then
    DestroyWindow(FBar);
  if FPopUp <> 0 then
    DestroyMenu(FPopUp);
  if FFont <> 0 then
    DeleteObject(FFont);
  FPHPSource.Free;
  inherited;
end;

procedure THandler.Update(const Sender: IInterface);

  function AddItem(Parent: HTREEITEM; const Caption: String;
    Data: Integer = -1): HTREEITEM;
  var
    Item: TTVInsertStruct;
  begin
    Item.hParent := Parent;
    Item.item.mask := TVIF_TEXT or TVIF_PARAM;
    Item.item.pszText := PChar(Caption);
    Item.item.lParam := Data;
    Result := HTREEITEM(SendMessage(FTree, TVM_INSERTITEM, 0, LPARAM(@Item)));
  end;

const
  Vis: array[TPHPVarVisibility] of string = ('Public', 'Private', 'Protected');
  Abstr: array[Boolean] of string = ('', ' [abstrakt]');
  Final: array[Boolean] of string = ('', ' [final]');
  Stat: array[Boolean]  of string = ('', ' [statisch]');
var
  i, j, k, l: integer;
  Node, SubNode, ClassNode, InterfaceNode: HTREEITEM;
  ClassText: string;
begin
  with FPHPSource do
  begin
    TreeView_DeleteAllItems(FTree);
    if not FApp.CanEditDocuments then Exit;
    AddItem(nil, 'Parsen luft, bitte warten...');
    UpdateWindow(FTree);
    SourceText := FApp.GetActiveDocument.GetEditor.GetText;
    TreeView_DeleteAllItems(FTree);
    if (IncludeCount = 0) and (MethodCount = 0) and (InterfaceCount = 0) and
      (ClassCount = 0) and (ConstantCount = 0) then
      AddItem(nil, '(Keine PHP Elemente verfgbar)');
    if IncludeCount > 0 then
      Node := AddItem(nil, 'Include Dateien');
    for i := 0 to IncludeCount-1 do
      SubNode := AddItem(Node, Includes[i].Occurence, Includes[i].TextPos);
    if MethodCount > 0 then
      Node := AddItem(nil, 'Methoden ohne Klassenbezug');
    for i := 0 to MethodCount-1 do
    begin
      SubNode := AddItem(Node, Methods[i].Head, Methods[i].TextPos);
      for k := 0 to Methods[i].ParamCount-1 do
        AddItem(SubNode, Methods[i].Parameters[k]);
    end;
    if ConstantCount > 0 then
      Node := AddItem(nil, 'Konstanten');
    for i := 0 to ConstantCount-1 do
    begin
      SubNode := AddItem(Node, Constants[i].Name + ' = ' +
        Constants[i].Value, Constants[i].TextPos);
    end;
    if InterfaceCount > 0 then
      InterfaceNode := AddItem(nil, 'Interfaces');
    for i := 0 to InterfaceCount-1 do
    begin
      ClassNode := AddItem(InterfaceNode, Interfaces[i].Name,
        Interfaces[i].TextPos);
      for l := Integer(low(TPHPVarVisibility)) to Integer(high(TPHPVarVisibility)) do
      begin
        Node := AddItem(ClassNode, Vis[TPHPVarVisibility(l)]);
        for j := 0 to Interfaces[i].MethodCount-1 do
        begin
          if not (Interfaces[i].Methods[j].Visibility = TPHPVarVisibility(l)) then
            Continue;
          SubNode := AddItem(Node, Interfaces[i].Methods[j].Head,
            Interfaces[i].Methods[j].TextPos);
          for k := 0 to Interfaces[i].Methods[j].ParamCount-1 do
            AddItem(SubNode, Interfaces[i].Methods[j].Parameters[k]);
        end;
      end;
    end;
    for i := 0 to ClassCount-1 do
    begin
      if Classes[i].ExtendsClass <> '' then
        ClassText := Classes[i].Name + ' (' +
          Classes[i].ExtendsClass + ')' else
      if Classes[i].ImplementedInterface <> '' then
        ClassText := Classes[i].Name + ' [' +
          Classes[i].ImplementedInterface + ']' else
      ClassText := Classes[i].Name;
      ClassText := ClassText + Abstr[Classes[i].IsAbstractClass];
      ClassNode := AddItem(nil, ClassText, Classes[i].TextPos);
      Node := AddItem(ClassNode, 'Konstanten');
      for l := 0 to Classes[i].ConstantCount-1 do
        AddItem(Node, Classes[i].Constants[l].Name,
          Classes[i].Constants[l].TextPos);
      for l := Integer(low(TPHPVarVisibility)) to Integer(high(TPHPVarVisibility)) do
      begin
        Node := AddItem(ClassNode, Vis[TPHPVarVisibility(l)]);
        for j := 0 to Classes[i].MethodCount-1 do
        begin
          if not (Classes[i].Methods[j].Visibility = TPHPVarVisibility(l)) then
            Continue;
          SubNode := AddItem(Node, Classes[i].Methods[j].Head +
            Abstr[Classes[i].Methods[j].IsAbstract] +
            Final[Classes[i].Methods[j].IsFinal] +
            Stat[Classes[i].Methods[j].IsStatic],
            Classes[i].Methods[j].TextPos);
          for k := 0 to Classes[i].Methods[j].ParamCount-1 do
            AddItem(SubNode, Classes[i].Methods[j].Parameters[k]);
        end;
        for j := 0 to Classes[i].VariableCount-1 do
        begin
          if not (Classes[i].Variables[j].Visibility = TPHPVarVisibility(l)) then
            Continue;
          SubNode := AddItem(Node, Classes[i].Variables[j].Occurence +
            Stat[Classes[i].Variables[j].IsStatic],
            Classes[i].Variables[j].TextPos);
        end;
      end;
    end;
  end;
end;

function LoadPlugIn(const Parent: IApplication; var Plugin: IPlugin):
  WordBool; stdcall;
var
  Handler: THandler;
begin
  Handler := THandler.Create(Parent);
  try
    Plugin := TPlugin.Create(Handler);
    Result := True;
  except
    Result := False;
  end;
  Handler.Update(nil);
end;

// Export the function
exports
  LoadPlugIn name 'LoadPlugIn';


begin
end.
